@@ -20,7 +20,7 @@
 /*! \file
  *
  * \author Joshua Colp <jcolp@digium.com>
- * \author Andreas 'MacBrody' Broadmann <andreas.brodmann@gmail.com>
+ * \author Andreas 'MacBrody' Brodmann <andreas.brodmann@gmail.com>
  *
  * \brief RTP (Multicast and Unicast) Media Channel
  *
@@ -43,6 +43,7 @@
 #include "asterisk/causes.h"
 #include "asterisk/format_cache.h"
 #include "asterisk/multicast_rtp.h"
+#include "asterisk/dns_core.h"
 
 /* Forward declarations */
 static struct ast_channel *multicast_rtp_request(const char *type, struct ast_format_cap *cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *data, int *cause);
@@ -293,9 +294,23 @@ static struct ast_channel *unicast_rtp_request(const char *type, struct ast_form
 		ast_log(LOG_ERROR, "Destination is required for the 'UnicastRTP' channel\n");
 		goto failure;
 	}
+
 	if (!ast_sockaddr_parse(&address, args.destination, PARSE_PORT_REQUIRE)) {
-		ast_log(LOG_ERROR, "Destination '%s' could not be parsed\n", args.destination);
-		goto failure;
+	    int rc;
+	    char *host;
+	    char *port;
+
+	    rc = ast_sockaddr_split_hostport(args.destination, &host, &port, PARSE_PORT_REQUIRE);
+	    if (!rc) {
+	        ast_log(LOG_ERROR, "Unable to parse destination '%s' into host and port\n", args.destination);
+	        goto failure;
+	    }
+
+	    rc = ast_dns_resolve_ipv6_and_ipv4(&address, host, port);
+	    if (rc != 0) {
+	        ast_log(LOG_ERROR, "Unable to resolve host '%s'\n", host);
+	        goto failure;
+	    }
 	}
 
 	if (!ast_strlen_zero(args.options)
@@ -366,9 +381,9 @@ static struct ast_channel *unicast_rtp_request(const char *type, struct ast_form
 
 	ast_channel_tech_pvt_set(chan, instance);
 
+	ast_rtp_instance_get_local_address(instance, &local_address);
 	pbx_builtin_setvar_helper(chan, "UNICASTRTP_LOCAL_ADDRESS",
 		ast_sockaddr_stringify_addr(&local_address));
-	ast_rtp_instance_get_local_address(instance, &local_address);
 	pbx_builtin_setvar_helper(chan, "UNICASTRTP_LOCAL_PORT",
 		ast_sockaddr_stringify_port(&local_address));
 
@@ -432,4 +447,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "RTP Media Channel",
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.requires = "res_rtp_multicast",
 );
